#ifndef _DEVICE_H_
#define _DEVICE_H_
#include <stdio.h>
#include <vector>
#include <assert.h>
#include "../xtypes.h"
#include "devinfo.h"
#include "../debugout.h"

namespace xgm
{
  const double DEFAULT_CLOCK = 1789772.0;
  const int DEFAULT_RATE = 48000;

  /**
   * G~[^ŎgpfoCX̒
   */
  class IDevice
  {
  public:
    /**
     * foCX̃Zbg
     *
     * <P>
     * ̃\bȟĂяoC̃foCX͂ǂ̂悤ȃ\bȟĂ
     * oɑ΂ĂCsG[NĂ͂ȂȂDtɁC̃\b
     * hĂԈȑÓC̃\bh͈̓ؕۏ؂ȂĂǂB
     * </P>
     */
    virtual void Reset () = 0;

    /**
     * foCXւ̏
     * 
     * @param adr AhX
     * @param val ޒl
     * @param id  foCXʏD̃foCXIOT|[g鎞Ȃ
     * @return  true s false
     */
    virtual bool Write (UINT32 adr, UINT32 val, UINT32 id=0)=0;

    /**
     * foCXǂݍ
     *
     * @param adr AhX
     * @param val ǂݏol󂯎ϐD
     * @return  true s false
     */
    virtual bool Read (UINT32 adr, UINT32 & val, UINT32 id=0)=0;

    /**
     * eIvVݒ肷()
     */
    virtual void SetOption (int id, int val){};
    virtual ~IDevice() {};
  };

  /**
   * C^[tF[XF̃_O\ȃNX
   */
  class IRenderable
  {
  public:
    /**
     * ̃_O
     * 
     * @param b[2] ꂽf[^i[zD
     * b[0]`lCb[1]E`l̉f[^D
     * @return f[^̃TCYD1Ȃ烂mD2ȂXeID0͍sD
     */
    virtual UINT32 Render (INT32 b[2]) = 0;

    // When seeking, this replaces Render
    virtual void Skip () {}

    /**
     *  chip update/operation is now bound to CPU clocks
     *  Render() now simply mixes and outputs sound
     */
    virtual void Tick (UINT32 clocks) {}
    virtual ~IRenderable() {};
  };

  /**
   * `bv
   */
  class ISoundChip : public IDevice, virtual public IRenderable
  {
  public:
    /**
     * Soundchip clocked by M2 (NTSC = ~1.789MHz)
     */
    virtual void Tick (UINT32 clocks) = 0;

    /**
     * `bv̓NbNݒ
     *
     * @param clock g
     */
    virtual void SetClock (double clock) = 0;

    /**
     * [gݒ
     *
     * @param rate o͎g
     */
    virtual void SetRate (double rate) = 0;

    /**
     * Channel mask.
     */
    virtual void SetMask (int mask)=0;

    /**
     * Stereo mix.
     *   mixl = 0-256
     *   mixr = 0-256
     *     128 = neutral
     *     256 = double
     *     0 = nil
     *    <0 = inverted
     */
    virtual void SetStereoMix(int trk, xgm::INT16 mixl, xgm::INT16 mixr) = 0;

    /**
     * Track info for keyboard view.
     */
    virtual ITrackInfo *GetTrackInfo(int trk){ return NULL; }
    virtual ~ISoundChip() {};
  };

  /**
   * oX
   *
   * <P>
   * ̃foCXɁCZbgC݁Cǂݍݓu[hLXgD
   * <P>
   */
  class Bus : public IDevice
  {
  protected:
    std::vector < IDevice * > vd;
  public:
    /**
     * Zbg
     *
     * <P>
     * tĂSẴfoCX́CReset\bhĂяoD
     * ĂяóCfoCXtꂽɓD
     * </P>
     */
    void Reset ()
    {
      std::vector < IDevice * >::iterator it;
      for (it = vd.begin (); it != vd.end (); it++)
        (*it)->Reset ();
    }

    /**
     * SfoCX̎O
     */
    void DetachAll ()
    {
      vd.clear ();
    }

    /**
     * foCX̎t
     *
     * <P>
     * ̃oXɃfoCXtD
     * </P>
     *
     * @param d tfoCXւ̃|C^
     */
    void Attach (IDevice * d)
    {
      vd.push_back (d);
    }

    /**
     * 
     *
     * <P>
     * tĂSẴfoCX́CWrite\bhĂяoD
     * ĂяóCfoCXtꂽɓD
     * </P>
     */
    bool Write (UINT32 adr, UINT32 val, UINT32 id=0)
    {
      bool ret = false;
      std::vector < IDevice * >::iterator it;
      for (it = vd.begin (); it != vd.end (); it++)
        ret |= (*it)->Write (adr, val);
      return ret;
    }

    /**
     * ǂݍ
     *
     * <P>
     * tĂSẴfoCXRead\bhĂяoD
     * ĂяóCfoCXtꂽɓD
     * Al͗L(Read\bhtrueԋp)foCX
     * Ԃl̘_aD
     * </P>
     */
    bool Read (UINT32 adr, UINT32 & val, UINT32 id=0)
    {
      bool ret = false;
      UINT32 vtmp = 0;
      std::vector < IDevice * >::iterator it;

      val = 0;
      for (it = vd.begin (); it != vd.end (); it++)
      {
        if ((*it)->Read (adr, vtmp))
        {
          val |= vtmp;
          ret = true;
        }
      }
      return ret;
    }
  };

  /**
   * C[
   *
   * <P>
   * oXƎĂ邪Cǂݏ̓SfoCXɓ`dȂD
   * ŏɓǂݏɐfoCX𔭌_ŏID
   * </P>
   */
  class Layer : public Bus
  {
  protected:
  public:
    /**
     * 
     *
     * <P>
     * tĂfoCXWrite\bhĂяoD
     * ĂяóCfoCXtꂽɓD
     * WriteɐfoCX_ŏID
     * </P>
     */
    bool Write (UINT32 adr, UINT32 val, UINT32 id=0)
    {
      std::vector < IDevice * >::iterator it;
      for (it = vd.begin (); it != vd.end (); it++)
        if ((*it)->Write (adr, val))
          return true;
      return false;
    }

    /**
     * ǂݍ
     *
     * <P>
     * tĂfoCXRead\bhĂяoD
     * ĂяóCfoCXtꂽɓD
     * ReadɐfoCX_ŏID
     * </P>
     */
    bool Read (UINT32 adr, UINT32 & val, UINT32 id=0)
    {
      std::vector < IDevice * >::iterator it;
      val = 0;
      for (it = vd.begin (); it != vd.end (); it++)
        if ((*it)->Read (adr, val))
          return true;
      return false;
    }
  };

}

#endif
